package com.diagbot.facade;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.AuthServiceClient;
import com.diagbot.dto.*;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.entity.DoctorAdvice;
import com.diagbot.entity.HomeDiagnoseInfo;
import com.diagbot.entity.HomeOperationInfo;
import com.diagbot.entity.HomePage;
import com.diagbot.entity.MedCheckInfo;
import com.diagbot.entity.MedCrisisInfo;
import com.diagbot.entity.MedLisInfo;
import com.diagbot.entity.MedLisResult;
import com.diagbot.entity.MedPacsInfo;
import com.diagbot.entity.MedPacsResult;
import com.diagbot.entity.MedicalRecord;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.BehospitalInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EncrypDES;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.AlgorithmVO;
import com.diagbot.vo.AnalyzeApiVO;
import com.diagbot.vo.AnalyzeCdsVO;
import com.diagbot.vo.AnalyzeCodeVO;
import com.diagbot.vo.AnalyzeRunVO;
import com.diagbot.vo.AnalyzeVO;
import com.diagbot.vo.BehospitalPageVO;
import com.diagbot.vo.ExportQcresultVO;
import com.diagbot.vo.GetDetailVO;
import com.diagbot.vo.MedrecVo;
import com.diagbot.vo.QcResultAlgQueryVO;
import com.diagbot.vo.QcResultAlgVO;
import com.diagbot.vo.QueryVo;
import com.diagbot.vo.RecordContentVO;
import com.diagbot.vo.TaskVO;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: zhoutg
 * @time: 2020/4/12 11:39
 */
@Component
public class BehospitalInfoFacade extends BehospitalInfoServiceImpl {

    @Autowired
    QcClientFacade qcServiceClient;
    @Autowired
    DoctorAdviceFacade doctorAdviceFacade;
    @Autowired
    HomePageFacade homePageFacade;
    @Autowired
    private QcCasesEntryFacade qcCasesEntryFacade;
    @Autowired
    HomeDiagnoseInfoFacade homeDiagnoseInfoFacade;
    @Autowired
    HomeOperationInfoFacade homeOperationInfoFacade;
    @Autowired
    MedicalRecordFacade medicalRecordFacade;
    @Autowired
    BasHospitalInfoFacade basHospitalInfoFacade;
    @Autowired
    private AlgorithmFacade algorithmFacade;
    @Autowired
    SysDictionaryFacade sysDictionaryFacade;
    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;
    @Autowired
    QcModeFacade qcModeFacade;
    @Autowired
    AuthServiceClient authServiceClient;
    @Autowired
    QcCasesEntryPagedataFacade qcCasesEntryPagedataFacade;
    @Value("${encrypt.enable}")
    Boolean encryptFlag;
    @Autowired
    QcQuestionFacade qcQuestionFacade;
    @Autowired
    QcModelHospitalFacade qcModelHospitalFacade;
    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    MedLisInfoFacade medLisInfoFacade;
    @Autowired
    MedLisResultFacade medLisResultFacade;
    @Autowired
    MedPacsInfoFacade medPacsInfoFacade;
    @Autowired
    MedPacsResultFacade medPacsResultFacade;
    @Autowired
    MedCrisisInfoFacade medCrisisInfoFacade;
    @Autowired
    MedCheckInfoFacade medCheckInfoFacade;

    /**
     * 分页
     *
     * @param behospitalPageVO
     * @return
     */
    public IPage<BehospitalInfoDTO> pageFac(BehospitalPageVO behospitalPageVO) {
        behospitalPageSet(behospitalPageVO);
        IPage<BehospitalInfoDTO> res = getPage(behospitalPageVO);
        return res;
    }

    /**
     * 获取明细
     *
     * @param getDetailVO
     * @return
     */
    public Map<String, Object> getByBehospitalCode(GetDetailVO getDetailVO) {
        Map<String, Object> res = new HashMap<>(); // 返回结果
        Long hospitalId = Long.parseLong(SysUserUtils.getCurrentHospitalID());
        getDetailVO.setHospitalId(hospitalId);
        // 获取病历信息
        BehospitalInfo behospitalInfo = this.getOne(new QueryWrapper<BehospitalInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", getDetailVO.getHospitalId())
                .eq("behospital_code", getDetailVO.getBehospitalCode()), false
        );

        List<String> paramList = getDetailVO.getParamStr();// 参数列表

        if (ListUtil.isEmpty(paramList) || paramList.contains("beHospital")) {
            BehosDTO behosDTO = new BehosDTO();
            if (behospitalInfo == null) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该病历已删除!");
            }
            BeanUtil.copyProperties(behospitalInfo, behosDTO);
            res.put("beHospital", behosDTO);
        }

        if (ListUtil.isEmpty(paramList) || paramList.contains("result") || paramList.contains("resultSimple")) {
            // 获取结果主表信息
            QcResultDTO qcResultDTO = qcresultInfoFacade.getByBehospitalCode(getDetailVO);
            if (qcResultDTO == null) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该病历未评分!");
            }
            if (paramList.contains("resultSimple")) {
                qcResultDTO.setPageData("");
                qcResultDTO.setMenuData("");
            }
            res.put("result", qcResultDTO);
            //增加该病历是否核查状态
            Map<String, Object> mapAll = new HashMap<String, Object>();
            mapAll.put("is_deleted", IsDeleteEnum.N.getKey());
            mapAll.put("behospital_code", getDetailVO.getBehospitalCode());
            mapAll.put("hospital_id", hospitalId);
            mapAll.put("check_type", 0);
            QueryWrapper<MedCheckInfo> medCheckInfoQuerys = new QueryWrapper<>();
            medCheckInfoQuerys.allEq(mapAll);
            int count = medCheckInfoFacade.count(medCheckInfoQuerys);
            if (count > 0) {
                res.put("checkStatus", 1);
            } else {
                res.put("checkStatus", 0);
            }
            //增加该病案首页是否核查状态
            Map<String, Object> mapAllMr = new HashMap<String, Object>();
            mapAllMr.put("is_deleted", IsDeleteEnum.N.getKey());
            mapAllMr.put("behospital_code", getDetailVO.getBehospitalCode());
            mapAllMr.put("hospital_id", hospitalId);
            mapAllMr.put("check_type", 1);
            QueryWrapper<MedCheckInfo> medCheckInfoMr = new QueryWrapper<>();
            medCheckInfoMr.allEq(mapAllMr);
            int countMr = medCheckInfoFacade.count(medCheckInfoMr);
            if (countMr > 0) {
                res.put("mrStatus", 1);
            } else {
                res.put("mrStatus", 0);
            }
        }

        // 获取提示信息
        if (ListUtil.isEmpty(paramList) || paramList.contains("msg")) {
            AnalyzeVO analyzeVO = new AnalyzeVO();
            BeanUtil.copyProperties(getDetailVO, analyzeVO);
            List<MsgDTO> msgDTOList = getMsg(analyzeVO);
            if (ListUtil.isNotEmpty(msgDTOList)) {
                // 从qc_question_info的cases_entry_ids获取
                Map<String, Object> paramMap = new HashMap<>();
                paramMap.put("hospitalId", hospitalId);
                paramMap.put("casesEntryIds", msgDTOList.stream().map(r -> r.getCasesEntryId()).collect(Collectors.toList()));
                Map<Long, List<QuestionEntryDTO>> quesEntryMap = qcQuestionFacade.getByCaseEntryIdsFac(paramMap);
                for (MsgDTO msgDTO : msgDTOList) {
                    if (quesEntryMap.get(msgDTO.getCasesEntryId()) != null) {
                        msgDTO.setPageKeyList(quesEntryMap.get(msgDTO.getCasesEntryId())
                                .stream().map(r -> r.getId()).collect(Collectors.toList()));
                    }
                }
            }
            // 根据模块分组
            Map<String, List<MsgDTO>> msgMap = EntityUtil.makeEntityListMap(msgDTOList, "modelName");
            res.put("msg", msgMap);

            // 添加DRGS分组
            List<MsgDTO> msgDTOListDrgs = msgDTOList.stream().filter(r -> r.getDrgs().equals(1)).collect(Collectors.toList());
            Map<String, List<MsgDTO>> drgsMap = EntityUtil.makeEntityListMap(msgDTOListDrgs, "modelName");
            res.put("drgs", drgsMap);
        }

        return res;
    }

    /**
     * 处理数据(公共方法)
     *
     * @param hospitalId
     * @param analyzeVO
     * @return
     */
    public QueryVo dealCommonData(Long hospitalId, AnalyzeVO analyzeVO) {
        // 获取质控条目
        List<QcCasesEntryDTO> qcCasesEntryDTOList = qcCasesEntryFacade.getQcCasesEntry(analyzeVO);
        if (ListUtil.isEmpty(qcCasesEntryDTOList)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该病历无关联的质控条目!");
        }

        // 获取病历所有数据
        Map<String, Map<String, String>> dicMap = sysDictionaryFacade.getDictionaryWithKey(); // 获取字典信息
        // 获取文书信息
        RecordContentVO recordContentVO = new RecordContentVO();
        BeanUtil.copyProperties(analyzeVO, recordContentVO);
        List<RecordContentDTO> recordContentDTOList = medicalRecordFacade.getRecordContentFac(recordContentVO);
        String recTitle = "";
        // 解密数据
        if (encryptFlag) {
            try {
                EncrypDES encrypDES = new EncrypDES();
                for (RecordContentDTO recordContentDTO : recordContentDTOList) {
                    recTitle = recordContentDTO.getRecTitle();
                    recordContentDTO.setXmlText(encrypDES.decryptor(recordContentDTO.getXmlText()));
                }
            } catch (Exception e) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                        "解密错误!病历号=【" + analyzeVO.getBehospitalCode() + "】，医院文书名称=【" + recTitle + "】");
            }
        }

        Map<String, List<RecordContentDTO>> recMap = EntityUtil.makeEntityListMap(recordContentDTOList, "standModelName");

        // 获取医嘱
        List<DoctorAdvice> doctorAdviceList = doctorAdviceFacade.list(new QueryWrapper<DoctorAdvice>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId)
                .eq("behospital_code", analyzeVO.getBehospitalCode())
                .orderByAsc("da_start_date")
        );

        // 危急值
        List<MedCrisisInfo> medCrisisInfoList = medCrisisInfoFacade.list(new QueryWrapper<MedCrisisInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId)
                .eq("behospital_code", analyzeVO.getBehospitalCode())
        );

        // 获取病案首页
        HomePage homePage = homePageFacade.getOne(new QueryWrapper<HomePage>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId)
                .eq("behospital_code", analyzeVO.getBehospitalCode()), false
        );

        List<HomeDiagnoseInfo> homePageList = new ArrayList<>();
        List<HomeOperationInfo> homeOperationInfoList = new ArrayList<>();
        if (homePage != null) {
            // 获取首页出院诊断、病理诊断
            homePageList = homeDiagnoseInfoFacade.list(new QueryWrapper<HomeDiagnoseInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", hospitalId)
                    .eq("home_page_id", homePage.getHomePageId())
                    .orderByAsc("diagnose_order_no")
            );

            // 获取首页手术信息
            homeOperationInfoList = homeOperationInfoFacade.list(new QueryWrapper<HomeOperationInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", hospitalId)
                    .eq("home_page_id", homePage.getHomePageId())
                    .orderByAsc("operation_order_no")
            );
            analyzeVO.setHaveHomePage(1);
        } else {
            analyzeVO.setHaveHomePage(0);
        }

        // 拼接数据
        String hosCode = basHospitalInfoFacade.getHosCode(hospitalId);
        QueryVo queryVo = new QueryVo();
        queryVo.setCid(hosCode); // 设置医院编码
        queryVo.setHospitalId(String.valueOf(hospitalId));
        queryVo.setInputCatalogueMap(getInputCatalogueMap(qcCasesEntryDTOList)); // 设置质控条目编码
        List<MedrecVo> medrecVoList = new ArrayList<>(); // 设置内容

        // 添加住院病历信息
        List<BehospitalInfo> behospitalInfoList = this.list(new QueryWrapper<BehospitalInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId)
                .eq("behospital_code", analyzeVO.getBehospitalCode())
        );
        if (ListUtil.isNotEmpty(behospitalInfoList)) {
            MedrecVo medrecVo = new MedrecVo();
            medrecVo.setTitle("住院病历信息");
            Map<String, Object> content = new HashMap<>();
            content.put("content", behospitalInfoList);
            medrecVo.setContent(content);
            medrecVoList.add(medrecVo);

            queryVo.setBehospitalInfo(behospitalInfoList.get(0));
        }


    // 化验
       // List<MedLisResult> medLisResultList = getMedLisResultList(behospitalInfoList.get(0));
    //使用sql进行关联优化数据查询
        List<MedLisResultDTO> medLisResultListDTO = medLisResultFacade.getMedLisResultList(behospitalInfoList.get(0));
        List<MedLisResult> medLisResultList =new ArrayList<>();
        for (MedLisResultDTO medLisResultSingle : medLisResultListDTO) {
            MedLisResult medLisResult = new MedLisResult();
            BeanUtil.copyProperties(medLisResultSingle,medLisResult);
            medLisResult.setItemName( medLisResultSingle.getRepName()+ "=" +medLisResultSingle.getReportName());
            medLisResultList.add(medLisResult);
        }

        // 辅检
       // List<MedPacsResult> medPacsResultList = getMedPacsResultList(behospitalInfoList.get(0));
      //使用sql进行关联优化数据查询
        List<MedPacsResult> medPacsResultList = medPacsResultFacade.getMedPacsResultList(behospitalInfoList.get(0));

        // 会诊记录
        addDataWithKey("会诊", recMap, medrecVoList,
                Arrays.asList("会诊记录", "会诊结果单", "会诊申请单"));

        // 手术
        addDataWithKey("手术", recMap, medrecVoList,
                Arrays.asList("术后首次病程及谈话记录", "手术记录",
                        "术前讨论、术前小结", "手术知情同意书", "手术安全核查表"));

        // 转科
        addDataWithKey("转科", recMap, medrecVoList,
                Arrays.asList("转入记录", "转出记录"));

        addData("入院记录", recMap, medrecVoList);
        addData("病危通知书", recMap, medrecVoList);
        addData("阶段小结", recMap, medrecVoList);
        addData("抢救记录", recMap, medrecVoList);
        addData("查房记录", recMap, medrecVoList);
        addData("首次病程录", recMap, medrecVoList);
        addData("输血/血制品病程记录", recMap, medrecVoList);
        addData("输血后效果评价", recMap, medrecVoList);
        addData("死亡病例讨论记录", recMap, medrecVoList);
        addData("死亡记录", recMap, medrecVoList);
        addData("危急值记录", recMap, medrecVoList);
        addData("出院小结", recMap, medrecVoList);
        addData("疑难病例讨论记录", recMap, medrecVoList);
        addData("输血后效果评价", recMap, medrecVoList);
        addData("病理检验送检单", recMap, medrecVoList);
        addData("日常病程录", recMap, medrecVoList);

        addDataWithInnerKey("知情同意书", recMap, medrecVoList);
        addDataWithInnerKey("谈话告知书", recMap, medrecVoList);

        // 医嘱
        if (ListUtil.isNotEmpty(doctorAdviceList)) {
            addDataWithKeyConvert("医嘱信息", doctorAdviceList, medrecVoList,
                    dicMap.get("1"), Arrays.asList("daStartDate", "daStopDate"));
        }

        // 危急值
        if (ListUtil.isNotEmpty(medCrisisInfoList)) {
            addDataWithKeyConvert("危急值", medCrisisInfoList, medrecVoList,
                    dicMap.get("20"), Arrays.asList("sendDate", "repDate"));
        }

        // 化验
        if (ListUtil.isNotEmpty(medLisResultList)) {
            addDataWithKeyConvert("化验", medLisResultList, medrecVoList,
                    dicMap.get("21"), new ArrayList<>());
        }

        // 辅检
        if (ListUtil.isNotEmpty(medPacsResultList)) {
            addDataWithKeyConvert("辅检", medPacsResultList, medrecVoList,
                    dicMap.get("22"), new ArrayList<>());
        }

        // 病案首页
        if (homePage != null) {
            addDataWithFirstPage("病案首页", homePage, medrecVoList, dicMap,
                    homePageList, homeOperationInfoList);
        }

        queryVo.setMedrec(medrecVoList);
        return queryVo;
    }

   /* private List<MedLisResult> getMedLisResultList(BehospitalInfo behospitalInfo) {
        String behospitalDate = null, leaveHospitalDate = null;
        try {
            behospitalDate = DateUtil.format(behospitalInfo.getBehospitalDate(), DateUtil.DATE_TIME_FORMAT);
            leaveHospitalDate = DateUtil.format(behospitalInfo.getLeaveHospitalDate(), DateUtil.DATE_TIME_FORMAT);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        if (StringUtil.isBlank(behospitalDate)) {
            return null;
        }
        List<MedLisResult> medLisResultList = null;
        QueryWrapper<MedLisInfo> medLisInfoQe = new QueryWrapper<>();
        medLisInfoQe.eq("is_deleted", IsDeleteEnum.N.getKey());
        medLisInfoQe.eq("hospital_id", behospitalInfo.getHospitalId());
        medLisInfoQe.like("behospital_code", behospitalInfo.getFileCode());
        medLisInfoQe.isNotNull("rep_name");
        medLisInfoQe.isNotNull("check_date");
        medLisInfoQe.apply("check_date!=''");
        medLisInfoQe.apply(
                "date_format (check_date,'%Y-%m-%d %H:%i:%s') >= date_format('"
                        + behospitalDate
                        + "','%Y-%m-%d %H:%i:%s')"
        );
        if (StringUtil.isNotBlank(behospitalInfo.getIsPlacefile())
                && behospitalInfo.getIsPlacefile().equals("1")
                && StringUtil.isNotBlank(leaveHospitalDate)) {
            medLisInfoQe.apply(
                    "date_format (check_date,'%Y-%m-%d %H:%i:%s') <= date_format('"
                            + leaveHospitalDate
                            + "','%Y-%m-%d %H:%i:%s')"
            );
        }
        medLisInfoQe.select("rep_no", "rep_name");
        List<MedLisInfo> medLisInfoList = medLisInfoFacade.list(medLisInfoQe);
        if (ListUtil.isNotEmpty(medLisInfoList)) {
            Map<String, String> repNoRepNameMap = medLisInfoList.stream().collect(Collectors.toMap(i -> i.getRepNo(), i -> i.getRepName()));
            medLisResultList = medLisResultFacade.list(new QueryWrapper<MedLisResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", behospitalInfo.getHospitalId())
                    .like("behospital_code", behospitalInfo.getFileCode())
                    .isNotNull("item_name")
                    .in("rep_no", medLisInfoList.stream().map(i -> i.getRepNo()).collect(Collectors.toList()))
            );
            medLisResultList.forEach(medLisResult -> {
                medLisResult.setItemName(repNoRepNameMap.get(medLisResult.getRepNo()) + "=" + medLisResult.getItemName());
            });
        }
        return medLisResultList;
    }*/

   /* private List<MedPacsResult> getMedPacsResultList(BehospitalInfo behospitalInfo) {
        String behospitalDate = null, leaveHospitalDate = null;
        try {
            behospitalDate = DateUtil.format(behospitalInfo.getBehospitalDate(), DateUtil.DATE_TIME_FORMAT);
            leaveHospitalDate = DateUtil.format(behospitalInfo.getLeaveHospitalDate(), DateUtil.DATE_TIME_FORMAT);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        if (StringUtil.isBlank(behospitalDate)) {
            return null;
        }
        List<MedPacsResult> medPacsResultList = null;
        QueryWrapper<MedPacsInfo> medPacsInfoQe = new QueryWrapper<>();
        medPacsInfoQe.eq("is_deleted", IsDeleteEnum.N.getKey());
        medPacsInfoQe.eq("hospital_id", behospitalInfo.getHospitalId());
        medPacsInfoQe.like("behospital_code", behospitalInfo.getFileCode());
        medPacsInfoQe.isNotNull("check_date");
        medPacsInfoQe.apply("check_date!=''");
        medPacsInfoQe.apply(
                "date_format (check_date,'%Y-%m-%d %H:%i:%s') >= date_format('"
                        + behospitalDate
                        + "','%Y-%m-%d %H:%i:%s')"
        );
        if (StringUtil.isNotBlank(behospitalInfo.getIsPlacefile())
                && behospitalInfo.getIsPlacefile().equals("1")
                && StringUtil.isNotBlank(leaveHospitalDate)) {
            medPacsInfoQe.apply(
                    "date_format (check_date,'%Y-%m-%d %H:%i:%s') <= date_format('"
                            + leaveHospitalDate
                            + "','%Y-%m-%d %H:%i:%s')"
            );
        }
        List<MedPacsInfo> medPacsInfoList = medPacsInfoFacade.list(medPacsInfoQe);
        if (ListUtil.isNotEmpty(medPacsInfoList)) {
            medPacsResultList = medPacsResultFacade.list(new QueryWrapper<MedPacsResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", behospitalInfo.getHospitalId())
                    .like("behospital_code", behospitalInfo.getFileCode())
                    .in("rep_no", medPacsInfoList.stream().map(i -> i.getRepNo()).collect(Collectors.toList()))
            );
        }
        return medPacsResultList;
    }*/

    /**
     * 评分-终末质控
     *
     * @param analyzeVO
     * @return
     */
    public AnalyzeDTO analyze(AnalyzeVO analyzeVO) {
        Long hospitalId = analyzeVO.getHospitalId();
        if (!analyzeVO.getIsTask()) {
            hospitalId = Long.parseLong(SysUserUtils.getCurrentHospitalID());
        }
        //        Long hospitalId = 1L; // 写死
        analyzeVO.setHospitalId(hospitalId);

        // 处理公共数据
        QueryVo queryVo = dealCommonData(hospitalId, analyzeVO);
        queryVo.setUseCrfCache(analyzeVO.isUseCrfCache());
        //  调用质控接口
        Response<OutputInfo> response = qcServiceClient.extract(queryVo);
        if (response == null || response.getData() == null) {
            if (null == response) {
                throw new CommonException(CommonErrorCode.RPC_ERROR, "远程质控接口没有返回数据!【" + analyzeVO.getBehospitalCode() + "】");
            } else {
                throw new CommonException(CommonErrorCode.RPC_ERROR,
                        "远程质控接口没有返回数据!【" + analyzeVO.getBehospitalCode() + "】错误原因:" + response.getMsg());
            }
        }
        OutputInfo outputInfo = response.getData();
        // 质控编码列表
        List<String> codeList = new ArrayList<>();
        // code和info的映射map
        Map<String, String> codeToInfoMap = new LinkedHashMap<>();
        // 对codeList 和 codeToInfoMap进行赋值
        setCodeData(outputInfo, codeList, codeToInfoMap);

        // 计算分值并保存结果至数据库
        Map<String, Object> resMap = calScoreAndSave(outputInfo, codeList, codeToInfoMap, analyzeVO, queryVo.getBehospitalInfo().getIsPlacefile());
        AlgorithmDTO algorithmDTO = (AlgorithmDTO)resMap.get("algorithmDTO");
        Date date = (Date)resMap.get("date");

        // 返回提示信息
        //        List<MsgDTO> msgDTOList = getMsg(analyzeVO);
        //        Map<String, List<MsgDTO>> msgMap = EntityUtil.makeEntityListMap(msgDTOList, "modelName");

        //返回参数组装
        AnalyzeDTO analyzeDTO = new AnalyzeDTO();
        analyzeDTO.setBehospitalCode(analyzeVO.getBehospitalCode());
        analyzeDTO.setIsSuccess(true);
        analyzeDTO.setGradeType(1);
        analyzeDTO.setGradeTypeName("机器");
        analyzeDTO.setScoreRes(algorithmDTO.getScore());
        analyzeDTO.setLevel(algorithmDTO.getLevel());
        analyzeDTO.setGradeTime(date);
        return analyzeDTO;
    }


    /**
     * 拼接数据
     *
     * @param key
     * @param recMap
     * @param medrecVoList
     */
    public void addData(String key, Map<String, List<RecordContentDTO>> recMap, List<MedrecVo> medrecVoList) {
        List<RecordContentDTO> list = recMap.get(key);
        Map<String, List<String>> listMap = new HashMap<>();
        if (ListUtil.isNotEmpty(list)) {
            MedrecVo medrecVo = new MedrecVo();
            medrecVo.setTitle(key);
            Map<String, Object> content = new HashMap<>();
            content.put("content", list);
            medrecVo.setContent(content);
            medrecVoList.add(medrecVo);
        }
    }


    /**
     * 拼接数据（例如：手术）
     *
     * @param key
     * @param recMap
     * @param medrecVoList
     */
    public void addDataWithKey(String key, Map<String, List<RecordContentDTO>> recMap,
                               List<MedrecVo> medrecVoList, List<String> keyList) {
        MedrecVo medrecVo = new MedrecVo();
        medrecVo.setTitle(key);
        Map<String, Object> content = new HashMap<>();
        Map<String, Object> listMap = new HashMap<>();
        Boolean flag = false;
        for (String k : keyList) {
            if (ListUtil.isNotEmpty(recMap.get(k))) {
                listMap.put(k, recMap.get(k));
                flag = true;
            }
        }
        if (flag) {
            // 会诊记录特殊处理，有会诊申请单，没有会诊结果单，复制一份到会诊结果单
            if ("会诊记录".equals(key) && listMap.get("会诊申请单") != null
                    && listMap.get("会诊结果单") == null) {
                listMap.put("会诊结果单", listMap.get("会诊申请单"));
            }
            content.put("content", listMap);
            medrecVo.setContent(content);
            medrecVoList.add(medrecVo);
        }
    }


    /**
     * 拼接数据（例如：知情同意书、谈话告知书）
     *
     * @param key
     * @param recMap
     * @param medrecVoList
     */
    public void addDataWithInnerKey(String key, Map<String, List<RecordContentDTO>> recMap, List<MedrecVo> medrecVoList) {
        MedrecVo medrecVo = new MedrecVo();
        medrecVo.setTitle(key);
        Map<String, Object> content = new HashMap<>();
        Map<String, Object> listMap = new HashMap<>();
        List<RecordContentDTO> recordContentDTOList = recMap.get(key);
        if (ListUtil.isNotEmpty(recordContentDTOList)) {
            Map<String, List<RecordContentDTO>> keyMap =
                    EntityUtil.makeEntityListMap(recordContentDTOList, "recTitle");
            Set<String> keyList = keyMap.keySet();
            for (String k : keyList) {
                if (ListUtil.isNotEmpty(keyMap.get(k))) {
                    listMap.put(k, keyMap.get(k));
                }
            }
            content.put("content", listMap);
            medrecVo.setContent(content);
            medrecVoList.add(medrecVo);
        }
    }

    /**
     * 拼接数据,从数据字典获取信息转换
     *
     * @param key
     * @param list
     * @param medrecVoList
     * @param map
     */
    public <T> void addDataWithKeyConvert(String key, List<T> list,
                                          List<MedrecVo> medrecVoList, Map<String, String> map, List<String> dateList) {
        MedrecVo medrecVo = new MedrecVo();
        medrecVo.setTitle(key);
        Map<String, Object> content = new HashMap<>();
        List<String> contents = new ArrayList<>();

        for (T bean : list) {
            try {
                Map<String, String> res = new HashMap<>();
                Map<String, Object> objectMap = MapUtil.objectToMap(bean);
                for (String objKey : objectMap.keySet()) {
                    if (map.get(objKey) != null) {
                        if (dateList.contains(objKey)) {
                            Date date = (Date) objectMap.get(objKey);
                            res.put(map.get(objKey), DateUtil.formatDateTime(date).replaceAll("null", ""));
                        } else {
                            res.put(map.get(objKey), String.valueOf(objectMap.get(objKey)).replaceAll("null", ""));
                        }
                    }
                }
                String jsonStr = JSON.toJSONString(res);
                contents.add(jsonStr);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        content.put("content", contents);
        medrecVo.setContent(content);
        medrecVoList.add(medrecVo);
    }


    /**
     * 对象key转换
     *
     * @param dateList
     * @param dicMap
     * @param objectMap
     * @return
     */
    public Map<String, Object> objcetKeyConvert(List<String> dateList, Map<String, String> dicMap, Map<String, Object> objectMap) {
        Map<String, Object> res = new HashMap<>();
        for (String objKey : objectMap.keySet()) {
            if (dicMap.containsKey(objKey)) {
                if (dateList.contains(objKey)) {
                    Date date = (Date) objectMap.get(objKey);
                    res.put(dicMap.get(objKey), DateUtil.formatDateTime(date).replaceAll("null", ""));
                } else {
                    res.put(dicMap.get(objKey), String.valueOf(objectMap.get(objKey)).replaceAll("null", ""));
                }
            }
        }
        return res;
    }

    /**
     * 拼接数据（病案首页),从数据字典获取信息转换
     *
     * @param key
     * @param homePage
     * @param medrecVoList
     * @param dicMap
     * @param homeDiagnoseInfoList
     * @param homeOperationInfoList
     */
    public void addDataWithFirstPage(String key, HomePage homePage, List<MedrecVo> medrecVoList, Map<String, Map<String, String>> dicMap,
                                     List<HomeDiagnoseInfo> homeDiagnoseInfoList, List<HomeOperationInfo> homeOperationInfoList) {
        MedrecVo medrecVo = new MedrecVo();
        medrecVo.setTitle(key);

        Map<String, String> map = dicMap.get("2"); // 病案首页
        Map<String, Object> content = new HashMap<>();
        try {
            Map<String, Object> objectMap = MapUtil.objectToMap(homePage);
            List<String> firstPageDate = Arrays.asList("birthday", "behospitalDate", "leaveHospitalDate", "qcDate");
            for (String objKey : objectMap.keySet()) {
                if (map.containsKey(objKey)) {
                    if (firstPageDate.contains(objKey)) {
                        Date date = (Date) objectMap.get(objKey);
                        if ("birthday".equals(objKey)) {
                            content.put(map.get(objKey), DateUtil.formatDate(date));
                        } else {
                            content.put(map.get(objKey), DateUtil.formatDateTime(date));
                        }
                    } else {
                        content.put(map.get(objKey), String.valueOf(objectMap.get(objKey)).replaceAll("null", ""));
                    }
                }
            }

            // 病案首页手术信息
            if (ListUtil.isNotEmpty(homeOperationInfoList)) {
                Map<String, String> dic3 = dicMap.get("3"); // 病案首页手术信息
                List<Map> mapList = new ArrayList<>();
                List<String> operatDate = Arrays.asList("operationDate");
                for (HomeOperationInfo homeOperationInfo : homeOperationInfoList) {
                    try {
                        Map operatMap = objcetKeyConvert(operatDate, dic3, MapUtil.objectToMap(homeOperationInfo));
                        mapList.add(operatMap);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                content.put("手术信息", JSON.toJSONString(mapList));
            }

            // 病案首页诊断信息
            if (ListUtil.isNotEmpty(homeDiagnoseInfoList)) {
                Map<String, String> dic4 = dicMap.get("4"); // 病案首页诊断信息
                List<Map> mapList = new ArrayList<>();
                List<String> diagnoseDate = Arrays.asList("");
                for (HomeDiagnoseInfo homeDiagnoseInfo : homeDiagnoseInfoList) {
                    try {
                        Map objectMapDiag = objcetKeyConvert(diagnoseDate, dic4, MapUtil.objectToMap(homeDiagnoseInfo));
                        mapList.add(objectMapDiag);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                content.put("出院诊断", JSON.toJSONString(mapList));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        medrecVo.setContent(content);
        medrecVoList.add(medrecVo);
    }


    /**
     * 获取质控条目编码
     *
     * @param qcCasesEntryDTOList
     * @return
     */
    public Map<String, Map<String, String>> getInputCatalogueMap(List<QcCasesEntryDTO> qcCasesEntryDTOList) {
        Map<String, Map<String, String>> res = new HashMap<>();
        for (QcCasesEntryDTO qc : qcCasesEntryDTOList) {
            Map<String, String> map = new HashMap<>();
            map.put("code", qc.getCode());
            map.put("precond", null);
            map.put("name", qc.getMsg());
            res.put(qc.getCode(), map);
        }
        return res;
    }


    /**
     * 处理定时任务入参
     *
     * @param param
     * @param taskVO
     */
    public void dealTaskVO(String param, TaskVO taskVO) {
        if (StringUtil.isNotBlank(param)) {
            String[] hos = param.split(",");
            List<Long> hosptialIdList = new ArrayList<>();
            for (String id : hos) {
                hosptialIdList.add(Long.parseLong(id));
            }
            taskVO.setHosptialIdList(hosptialIdList);
        }
    }

    /**
     * 评分
     */
    public void execute(TaskVO taskVO, String param) {
        // 处理医院信息
        dealTaskVO(param, taskVO);
        // 类型过滤(1：出院时间比当前时间早4天前的病历（例如：长兴病历）)
        if (taskVO.getType() != null && taskVO.getType() == 1) {
            Date date = DateUtil.addDay(DateUtil.now(), -4);
            taskVO.setLeaveDate(date);
        }
        // 近3天不含当天的病历强制评分更新
        if (taskVO != null && ListUtil.isNotEmpty(taskVO.getIndividuation()) && taskVO.getIndividuation().contains(1)) {
            // 4天前最早时间
            taskVO.setStartLeaveDate(DateUtil.getFirstTimeOfDay(DateUtil.addDay(DateUtil.now(), -4)));
            // 昨天最迟时间
            taskVO.setEndLeaveDate(DateUtil.getLastTimeOfDay(DateUtil.addDay(DateUtil.now(), -1)));
        }
        List<BehospitalInfo> behospitalInfoList = getNoGrade(taskVO);
        for (BehospitalInfo bean : behospitalInfoList) {
            AnalyzeVO analyzeVO = new AnalyzeVO();
            analyzeVO.setIsTask(true);
            analyzeVO.setHospitalId(bean.getHospitalId());
            analyzeVO.setBehospitalCode(bean.getBehospitalCode());
            analyzeVO.setUseCrfCache(taskVO.isUseCrfCache());
            authServiceClient.analyze_rpc(analyzeVO);
        }
    }


    // /**
    //  * 对出院日期之前3天的未评分的病历进行评分
    //  */
    // public void execute2(TaskVO taskVO) {
    //     List<BehospitalInfo> behospitalInfoList = getNoGrade(taskVO);
    //     for (BehospitalInfo bean : behospitalInfoList) {
    //         AnalyzeVO analyzeVO = new AnalyzeVO();
    //         analyzeVO.setIsTask(true);
    //         analyzeVO.setHospitalId(bean.getHospitalId());
    //         analyzeVO.setBehospitalCode(bean.getBehospitalCode());
    //         authServiceClient.analyze_rpc(analyzeVO);
    //     }
    // }

    /**
     * 评分api
     *
     * @param analyzeApiVO
     * @return
     */
    public Map<String, Object> analyzeApi(AnalyzeApiVO analyzeApiVO) {
        AnalyzeVO analyzeVO = new AnalyzeVO();
        BeanUtil.copyProperties(analyzeApiVO, analyzeVO);
        // 评分
        analyze(analyzeVO);
        // 获取结果
        GetDetailVO getDetailVO = new GetDetailVO();
        getDetailVO.setHospitalId(analyzeApiVO.getHospitalId());
        getDetailVO.setBehospitalCode(analyzeApiVO.getBehospitalCode());
        getDetailVO.setNeedGroup(analyzeApiVO.getNeedGroup());
        return getByBehospitalCodeApi(getDetailVO);
    }


    /**
     * 获取明细api
     *
     * @param getDetailVO
     * @return
     */
    public Map<String, Object> getByBehospitalCodeApi(GetDetailVO getDetailVO) {
        Map<String, Object> res = new HashMap<>(); // 返回结果
        Long hospitalId = getDetailVO.getHospitalId();
        // 获取病历信息
        BehospitalInfo behospitalInfo = this.getOne(new QueryWrapper<BehospitalInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", getDetailVO.getHospitalId())
                .eq("behospital_code", getDetailVO.getBehospitalCode()), false
        );

        BehosDTO behosDTO = new BehosDTO();
        if (behospitalInfo == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "该病历不存在!【hospitalId=" + hospitalId + "】【behospitalCode="
                            + getDetailVO.getBehospitalCode() + "】");
        }
        BeanUtil.copyProperties(behospitalInfo, behosDTO);

        // 获取结果主表信息
        QcResultDTO qcResultDTO = qcresultInfoFacade.getByBehospitalCode(getDetailVO);
        if (qcResultDTO == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "该病历未评分!【hospitalId=" + hospitalId + "】【behospitalCode="
                            + getDetailVO.getBehospitalCode() + "】");
        }
        QcResultApiDTO qcResultApiDTO = new QcResultApiDTO();
        BeanUtil.copyProperties(qcResultDTO, qcResultApiDTO);
        res.put("result", qcResultApiDTO);

        // 获取提示信息
        AnalyzeVO analyzeVO = new AnalyzeVO();
        BeanUtil.copyProperties(getDetailVO, analyzeVO);
        List<MsgDTO> msgDTOList = getMsg(analyzeVO);
        List<MsgApiDTO> msgApiDTOList = BeanUtil.listCopyTo(msgDTOList, MsgApiDTO.class);
        if (getDetailVO.getNeedGroup() == 0) {
            res.put("details", msgApiDTOList);
        } else {
            Map<String, List<MsgApiDTO>> msgMap = EntityUtil.makeEntityListMap(msgApiDTOList, "modelName");
            res.put("details", msgMap);
        }
        return res;
    }

    /**
     * @param behospitalPageVO
     * @return
     */
    public IPage<BehospitalInfoDTO> pagePerson(BehospitalPageVO behospitalPageVO) {
        behospitalPageSet(behospitalPageVO);
        behospitalPageVO.setUserId(Long.parseLong(SysUserUtils.getCurrentPrincipleID()));
        IPage<BehospitalInfoDTO> res = getPageByPerson(behospitalPageVO);
        return res;
    }

    /**
     * @param behospitalPageVO
     * @return
     */
    public IPage<BehospitalInfoDTO> pageGroup(BehospitalPageVO behospitalPageVO) {
        behospitalPageSet(behospitalPageVO);
        behospitalPageVO.setUserId(Long.parseLong(SysUserUtils.getCurrentPrincipleID()));
        IPage<BehospitalInfoDTO> res = getPageByGroup(behospitalPageVO);
        return res;
    }

    public IPage<BehospitalInfoDTO> pageDept(BehospitalPageVO behospitalPageVO) {
        behospitalPageSet(behospitalPageVO);
        behospitalPageVO.setUserId(Long.parseLong(SysUserUtils.getCurrentPrincipleID()));
        IPage<BehospitalInfoDTO> res = getPageByDept(behospitalPageVO);
        return res;
    }

    /**
     * 评分-运行质控
     *
     * @param analyzeRunVO
     * @return
     */
    public AnalyzeRunDTO analyzeRun(AnalyzeRunVO analyzeRunVO) {
        AnalyzeVO analyzeVO = new AnalyzeVO();
        BeanUtil.copyProperties(analyzeRunVO, analyzeVO);
        Long hospitalId = analyzeVO.getHospitalId();

        // 处理公共数据
        QueryVo queryVo = dealCommonData(hospitalId, analyzeVO);
        //  调用质控接口
        Response<OutputInfo> response = qcServiceClient.extract(queryVo);
        if (response == null || response.getData() == null) {
            if (null == response) {
                throw new CommonException(CommonErrorCode.RPC_ERROR, "远程质控接口没有返回数据!【" + analyzeVO.getBehospitalCode() + "】");
            } else {
                throw new CommonException(CommonErrorCode.RPC_ERROR,
                        "远程质控接口没有返回数据!【" + analyzeVO.getBehospitalCode() + "】错误原因:" + response.getMsg());
            }
        }
        OutputInfo outputInfo = response.getData();
        // 质控编码列表
        List<String> codeList = new ArrayList<>();
        // code和info的映射map
        Map<String, String> codeToInfoMap = new LinkedHashMap<>();
        // 对codeList 和 codeToInfoMap进行赋值
        setCodeData(outputInfo, codeList, codeToInfoMap);

        // 如果是1，说明已是终末质控，不再保存质控结果数据；如果是0，则保存质控结果数据
        if ("0".equals(analyzeRunVO.getIsPlacefile())) {
            // 计算分值并保存结果至数据库
            Map<String, Object> resMap = calScoreAndSave(outputInfo, codeList, codeToInfoMap, analyzeVO, queryVo.getBehospitalInfo().getIsPlacefile());
        }

        // 返回缺陷提示信息
        List<MsgDTO> msgDTOList = new ArrayList<>();
        if (ListUtil.isNotEmpty(codeList)) {
            AnalyzeCodeVO analyzeCodeVO = new AnalyzeCodeVO();
            analyzeCodeVO.setCodeList(codeList);
            analyzeCodeVO.setHospitalId(hospitalId);
            analyzeCodeVO.setModeId(analyzeRunVO.getModeId());

            // 获取缺陷条目信息
            msgDTOList = getMsgByEntryCode(analyzeCodeVO);
            // 设置info信息
            if (codeToInfoMap != null && !codeToInfoMap.isEmpty()) {
                for (MsgDTO msgDTO : msgDTOList) {
                    String info = codeToInfoMap.get(msgDTO.getCode());
                    if (StringUtil.isNotBlank(info)) {
                        msgDTO.setInfo(info);
                    }
                }
            }
        }

        //返回参数组装
        AnalyzeRunDTO analyzeRunDTO = new AnalyzeRunDTO();
        // analyzeRunDTO.setScoreRes(algorithmDTO.getScore());
        // analyzeRunDTO.setLevel(algorithmDTO.getLevel());
        analyzeRunDTO.setMsgDTOList(msgDTOList);
        return analyzeRunDTO;
    }

    private void behospitalPageSet(BehospitalPageVO behospitalPageVO) {
        //入参验证
        //入院时间
        if (null != behospitalPageVO && null != behospitalPageVO.getBehosDateStart()) {
            behospitalPageVO.setBehosDateStart(DateUtil.getFirstTimeOfDay(behospitalPageVO.getBehosDateStart()));
        }
        if (null != behospitalPageVO && null != behospitalPageVO.getBehosDateEnd()) {
            behospitalPageVO.setBehosDateEnd(DateUtil.getFirstTimeOfDay(DateUtil.addDay(behospitalPageVO.getBehosDateEnd(), 1)));
        }
        if (null != behospitalPageVO && null != behospitalPageVO.getBehosDateStart() && null != behospitalPageVO.getBehosDateEnd()) {
            if (DateUtil.after(behospitalPageVO.getBehosDateStart(), behospitalPageVO.getBehosDateEnd())) {
                throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "入院时间的开始时间必须小于结束时间！");
            }
        }
        //出院时间
        if (null != behospitalPageVO && null != behospitalPageVO.getLeaveHosDateStart()) {
            behospitalPageVO.setLeaveHosDateStart(DateUtil.getFirstTimeOfDay(behospitalPageVO.getLeaveHosDateStart()));
        }
        if (null != behospitalPageVO && null != behospitalPageVO.getLeaveHosDateEnd()) {
            behospitalPageVO.setLeaveHosDateEnd(DateUtil.getFirstTimeOfDay(DateUtil.addDay(behospitalPageVO.getLeaveHosDateEnd(), 1)));
        }
        if (null != behospitalPageVO && null != behospitalPageVO.getLeaveHosDateStart() && null != behospitalPageVO.getLeaveHosDateEnd()) {
            if (DateUtil.after(behospitalPageVO.getLeaveHosDateStart(), behospitalPageVO.getLeaveHosDateEnd())) {
                throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "出院时间的开始时间必须小于结束时间！");
            }
        }
        behospitalPageVO.setHospitalId(Long.parseLong(SysUserUtils.getCurrentHospitalID()));
//        behospitalPageVO.setIsPlacefile("1");
//        if (null != behospitalPageVO.getStatisticsType()
//                && null == behospitalPageVO.getLeaveHosDateStart()
//                && null == behospitalPageVO.getLeaveHosDateEnd()) {
//            String startDate = filterFacade.getStartDateStr(behospitalPageVO.getStatisticsType(), null);
//            String endDate = filterFacade.getEndDateStr(behospitalPageVO.getStatisticsType(), null);
//            behospitalPageVO.setLeaveHosDateStart(DateUtil.parseDate(startDate));
//            behospitalPageVO.setLeaveHosDateEnd(DateUtil.parseDate(endDate));
//        }
    }

    public void exportExcelFac(HttpServletResponse response) {
        List<ExportExcelDTO> res = this.exportExcel();
        String fileName = "mrqc";
        ExcelUtils.exportExcelUser(res, null, "病案质控", ExportExcelDTO.class, fileName + ".xls", response);
    }

    /**
     * 质控结果导出
     *
     * @param response
     * @param exportQcresultVO
     */
    public void exportQcresult(HttpServletResponse response, ExportQcresultVO exportQcresultVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        exportQcresultVO.setHospitalId(hospitalId);
        //时间设置
        exportQcresultVOSet(exportQcresultVO);
        List<ExportExcelDTO> res = this.exportQcresult(exportQcresultVO);
        String fileName = "抽查住院病历质量情况.xls";
        response.setContentType("text/html;charset=UTF-8");
        ExcelUtils.exportExcelUser(res, null, "sheet1", ExportExcelDTO.class, fileName, response);
    }

    /**
     * 科室质控结果导出
     *
     * @param response
     * @param exportQcresultVO
     */
    public void exportQcresultByDept(HttpServletResponse response, ExportQcresultVO exportQcresultVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        exportQcresultVO.setHospitalId(hospitalId);
        exportQcresultVO.setUserId(Long.valueOf(userId));
        //时间设置
        exportQcresultVOSet(exportQcresultVO);
        List<ExportExcelDTO> res = this.exportQcresultByDept(exportQcresultVO);
        String fileName = "抽查住院病历质量情况.xls";
        response.setContentType("text/html;charset=UTF-8");
        ExcelUtils.exportExcelUser(res, null, "sheet1", ExportExcelDTO.class, fileName, response);
    }

    /**
     * 医疗组质控结果导出
     *
     * @param response
     * @param exportQcresultVO
     */
    public void exportQcresultByGroup(HttpServletResponse response, ExportQcresultVO exportQcresultVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        exportQcresultVO.setHospitalId(hospitalId);
        exportQcresultVO.setUserId(Long.valueOf(userId));
        //时间设置
        exportQcresultVOSet(exportQcresultVO);
        List<ExportExcelDTO> res = this.exportQcresultByGroup(exportQcresultVO);
        String fileName = "抽查住院病历质量情况.xls";
        response.setContentType("text/html;charset=UTF-8");
        ExcelUtils.exportExcelUser(res, null, "sheet1", ExportExcelDTO.class, fileName, response);
    }

    private void exportQcresultVOSet(ExportQcresultVO exportQcresultVO){
        //入参验证
        long interval = 7 * 24 * 60 * 60 * 1000;
        //入院时间
        if (null != exportQcresultVO && null != exportQcresultVO.getBehosDateStart() && null != exportQcresultVO.getBehosDateEnd()) {
            Date startDate = exportQcresultVO.getBehosDateStart();
            Date endDate = exportQcresultVO.getBehosDateEnd();
            //时间间隔7天
            if (endDate.getTime() < startDate.getTime()) {
                throw new CommonException(CommonErrorCode.PARAM_ERROR, "截止时间不能小于起始时间");
            }
            if (endDate.getTime() - startDate.getTime() > interval) {
                throw new CommonException(CommonErrorCode.PARAM_ERROR, "统计区间不能大于7天");
            }
            exportQcresultVO.setBehosDateStart(DateUtil.getFirstTimeOfDay(exportQcresultVO.getBehosDateStart()));
            exportQcresultVO.setBehosDateEnd(DateUtil.getFirstTimeOfDay(DateUtil.addDay(exportQcresultVO.getBehosDateEnd(), 1)));
        }
        //出院时间
        if (null != exportQcresultVO && null != exportQcresultVO.getLeaveHosDateStart() && null != exportQcresultVO.getLeaveHosDateEnd()) {
            Date startDate = exportQcresultVO.getLeaveHosDateStart();
            Date endDate = exportQcresultVO.getLeaveHosDateEnd();
            //时间间隔7天
            if (endDate.getTime() < startDate.getTime()) {
                throw new CommonException(CommonErrorCode.PARAM_ERROR, "截止时间不能小于起始时间");
            }
            if (endDate.getTime() - startDate.getTime() > interval) {
                throw new CommonException(CommonErrorCode.PARAM_ERROR, "统计区间不能大于7天");
            }
            exportQcresultVO.setLeaveHosDateStart(DateUtil.getFirstTimeOfDay(exportQcresultVO.getLeaveHosDateStart()));
            exportQcresultVO.setLeaveHosDateEnd(DateUtil.getFirstTimeOfDay(DateUtil.addDay(exportQcresultVO.getLeaveHosDateEnd(), 1)));
        }
    }

    /**
     * 设置共用code数据
     *
     * @param outputInfo 质控出参数据
     * @param codeList 质控编码列表
     * @param codeToInfoMap 质控编码对应的info信息
     */
    public void setCodeData(OutputInfo outputInfo, List<String> codeList, Map<String, String> codeToInfoMap) {
        Map<String, Map<String, String>> codeMap = outputInfo.getResult();
        for (String key : codeMap.keySet()) {
            codeList.add(key);
            Map<String, String> mapInfo = codeMap.get(key);
            if (mapInfo != null && StringUtil.isNotBlank(mapInfo.get("info"))) {
                codeToInfoMap.put(key, mapInfo.get("info"));
            }
        }
    }


    /**
     * 计算分值并保存结果至数据库
     *
     * @param outputInfo 质控出参数据
     * @param codeList 质控编码列表
     * @param codeToInfoMap 质控编码对应的info信息
     * @param analyzeVO 入参
     * @return
     */
    public Map<String, Object> calScoreAndSave(OutputInfo outputInfo, List<String> codeList,
                                               Map<String, String> codeToInfoMap, AnalyzeVO analyzeVO,
                                               String isPlacefile) {
        Long hospitalId = analyzeVO.getHospitalId();
        List<QcResultAlgVO> qcResultAlgVOList = new ArrayList<>();
        if (ListUtil.isNotEmpty(codeList)) {
            // 根据质控结果获取质控条目
            QcResultAlgQueryVO qcResultAlgQueryVO = new QcResultAlgQueryVO();
            qcResultAlgQueryVO.setCodeList(codeList);
            qcResultAlgQueryVO.setHospitalId(hospitalId);
            qcResultAlgQueryVO.setIsPlacefile(isPlacefile);
            qcResultAlgVOList = qcCasesEntryFacade.getQcResultAlgVO(qcResultAlgQueryVO);
        }

        // 对info赋值
        for (QcResultAlgVO bean : qcResultAlgVOList) {
            String info = codeToInfoMap.get(bean.getCode());
            if (StringUtil.isNotBlank(info)) {
                bean.setInfo(info);
            }
        }

        // 评分
        AlgorithmVO algorithmVO = new AlgorithmVO();
        algorithmVO.setType(0);
        algorithmVO.setQcResultAlgVOList(qcResultAlgVOList);
        algorithmVO.setHospitalId(hospitalId);
        algorithmVO.setBehospitalCode(analyzeVO.getBehospitalCode());
        algorithmVO.setQcResultAlgVOList(qcResultAlgVOList);
        algorithmVO.setIsPlacefile(isPlacefile);
        AlgorithmDTO algorithmDTO = algorithmFacade.getAlgorithmRes(algorithmVO);

        //保存
        Map<String, Object> pageMap = outputInfo.getPageData();
        // 手动拼接数据【知情同意书】【谈话告知书】
        List<MedicalRecord> recordList = medicalRecordFacade.list(new QueryWrapper<MedicalRecord>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("behospital_code", analyzeVO.getBehospitalCode())
                .eq("hospital_id", hospitalId)
                .in("mode_id", Arrays.asList(53, 54, 16))
                .orderByAsc("rec_date")
        );
        List<MedicalRecordDTO> medicalRecordDTOList = BeanUtil.listCopyTo(recordList, MedicalRecordDTO.class);
        Map<Long, List<MedicalRecordDTO>> recordMap = EntityUtil.makeEntityListMap(medicalRecordDTOList, "modeId");
        List<MedicalRecordDTO> recordDTOList = new ArrayList<>();
        if (recordMap.get(53L) != null) {
            recordDTOList.addAll(recordMap.get(53L)); // 知情同意书
        }
        if (recordMap.get(16L) != null) {
            recordDTOList.addAll(recordMap.get(16L)); // 手术知情同意书
        }
        // 医嘱信息
        pageMap.put("医嘱信息", null);
        // 医嘱信息
        pageMap.put("检查信息", null);
        // 医嘱信息
        pageMap.put("检验信息", null);
        // 知情同意书 = 【知情同意书】 + 【手术知情同意书】
        pageMap.put("知情同意书", recordDTOList);
        pageMap.put("谈话告知书", recordMap.get(54L));

        String pageData = JSON.toJSONString(pageMap);
        // 获取菜单信息
        List<QcModeDTO> qcModeDTOList = qcModeFacade.getMenu(pageMap);
        String menuData = JSON.toJSONString(qcModeDTOList);

        // 保存评分结果信息
        Date date = qcresultInfoFacade.saveQcResult(algorithmDTO, algorithmVO, analyzeVO, pageData, menuData, analyzeVO.getIsTask(), null);

        // 返回结果信息
        Map<String, Object> map = new HashMap<>();
        map.put("algorithmDTO", algorithmDTO);
        map.put("date", date);
        return map;
    }

    /**
     * cds访问质控接口
     *
     * @param analyzeCdsVO
     * @return
     */
    public Map<String,String> analyzeCds(AnalyzeCdsVO analyzeCdsVO) {
        Map<String,String> ret = Maps.newHashMap();
        List<QcCasesEntryDTO> qcCasesEntryDTOList = qcCasesEntryFacade.getQcCasesEntryCds(analyzeCdsVO);
        Map<String,String> codeMsgMap = qcCasesEntryDTOList.stream().collect(Collectors.toMap(i->i.getCode(),i->i.getMsg()));

        QueryVo queryVo = new QueryVo();
        queryVo.setCid(basHospitalInfoFacade.getHosCode(analyzeCdsVO.getHospitalId())); // 设置医院编码
        queryVo.setHospitalId(String.valueOf(analyzeCdsVO.getHospitalId()));
        queryVo.setInputCatalogueMap(getInputCatalogueMap(qcCasesEntryDTOList)); // 设置质控条目编码
        queryVo.setUseCrfCache(false);

        List<MedrecVo> medrecVoList = new ArrayList<>(); // 设置内容

        MedrecVo outDepMedrecVo = new MedrecVo();
        outDepMedrecVo.setTitle("门诊");
        Map<String, Object> outDepContent = new HashMap<>();
        List<Map<String, String>> outDepMaps = Lists.newArrayList();
        Map<String, String> outDepMap = Maps.newHashMap();
        outDepMap.put("主诉", analyzeCdsVO.getChief());
        outDepMap.put("现病史", analyzeCdsVO.getSymptom());
        outDepMap.put("既往史", analyzeCdsVO.getPasts());
        outDepMaps.add(outDepMap);
        outDepContent.put("content", outDepMaps);
        outDepMedrecVo.setContent(outDepContent);
        medrecVoList.add(outDepMedrecVo);

        MedrecVo beInfoMedrecVo = new MedrecVo();
        beInfoMedrecVo.setTitle("住院病历信息");
        List<BehospitalInfo> behospitalInfoList = Lists.newArrayList();
        BehospitalInfo behospitalInfo = new BehospitalInfo();
        behospitalInfo.setBehospitalCode("cds-menzhen");
        behospitalInfoList.add(behospitalInfo);
        Map<String, Object> beInfoContent = new HashMap<>();
        beInfoContent.put("content", behospitalInfoList);
        beInfoMedrecVo.setContent(beInfoContent);
        medrecVoList.add(beInfoMedrecVo);
        queryVo.setBehospitalInfo(behospitalInfo);
        queryVo.setMedrec(medrecVoList);

        Response<OutputInfo> response = qcServiceClient.extract(queryVo);
        if (response == null || response.getData() == null) {
            if (null == response) {
                throw new CommonException(CommonErrorCode.RPC_ERROR, "远程质控接口没有返回数据!");
            } else {
                throw new CommonException(CommonErrorCode.RPC_ERROR, "远程质控接口没有返回数据！错误原因:" + response.getMsg());
            }
        }

        OutputInfo outputInfo = response.getData();
        outputInfo.getResult().keySet().forEach(key->{
            ret.put(codeMsgMap.get(key),outputInfo.getResult().get(key).get("info"));
        });

        return ret;
    }

}
