package com.diagbot.facade.data;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ABasDoctorInfoDTO;
import com.diagbot.entity.BasDoctorInfo;
import com.diagbot.service.impl.BasDoctorInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.ABasDoctorInfoVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class ABasDoctorInfoFacade extends BasDoctorInfoServiceImpl {
    @Autowired
    @Qualifier("basDoctorInfoServiceImpl")
    private BasDoctorInfoServiceImpl basDoctorInfoService;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 同步历史数据
     */
    public void executeDoctorPast() {
        String sql="select * from gi_userinfo";
        List<BasDoctorInfo> basDeptInfoList = tzDBConn.getBasDoctorInfo(sql);
        execute(basDeptInfoList);

    }
    /**
     * 同步前一天医生信息
     */
    public void executeDoctor() {
        String sql="select * from gi_userinfo";
        List<BasDoctorInfo> basDeptInfoList = tzDBConn.getBasDoctorInfo(sql);
        execute(basDeptInfoList);
    }
    /**
     * 从接口接收医生信息
     * @param list
     * @return
     */
    public RespDTO<List<ABasDoctorInfoDTO>> executeDoctor(List<ABasDoctorInfoVO> list) {
        if(list!=null && list.size()>0){
            List<BasDoctorInfo> basDoctorInfoList=Lists.newArrayList();
            List<ABasDoctorInfoDTO> basDoctorDtoList=Lists.newArrayList();
            basDoctorInfoList=BeanUtil.listCopyTo(list,BasDoctorInfo.class);
            basDoctorDtoList=BeanUtil.listCopyTo(list,ABasDoctorInfoDTO.class);
            //循环验证数据有效性
            for (BasDoctorInfo basDoctorInfo:basDoctorInfoList) {
                if("".equals(basDoctorInfo.getDoctorId())) {
                    return RespDTO.onError("请输入医生编码!");
                }else if(basDoctorInfo.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }
            }
            execute(basDoctorInfoList);
            return RespDTO.onSuc(basDoctorDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<BasDoctorInfo> basDoctorInfoList){
        long startTime=System.currentTimeMillis();
        List<BasDoctorInfo> addE=Lists.newArrayList();
        List<BasDoctorInfo> updateE=Lists.newArrayList();
        basDoctorInfoList.stream().forEach(s -> {
            BasDoctorInfo basDoctorInfo =this.baseMapper.selectOne(new QueryWrapper<BasDoctorInfo>()
                    .eq("doctor_id", s.getDoctorId())
                    .eq("hospital_id", s.getHospitalId()));
            if (basDoctorInfo != null) {
                s.setGmtModified(new Date());
                updateE.add(s);
            } else {
                s.setGmtCreate(new Date());
                addE.add(s);
            }
        });
        if(updateE.size()>0){
            basDoctorInfoService.updateBatchByKey(updateE);
        }
        if(addE.size()>0){
            basDoctorInfoService.saveBatch(addE);
        }

        long endTime=System.currentTimeMillis();
        System.out.println("程序运行时间： "+(endTime - startTime)+"ms");
    }
}
