package com.diagbot.facade.data;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ABehospitalInfoDTO;
import com.diagbot.entity.*;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.facade.QcAbnormalFacade;
import com.diagbot.facade.QcTypeFacade;
import com.diagbot.service.impl.BehospitalInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.ABehospitalInfoVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class ABehospitalInfoFacade extends BehospitalInfoServiceImpl {

    @Autowired
    @Qualifier("behospitalInfoServiceImpl")
    private BehospitalInfoServiceImpl behospitalInfoService;

    @Autowired
    private QcTypeFacade qcTypeFacade;

    @Autowired
    private AMedicalRecordFacade aMedicalRecordFacade;

    @Autowired
    private QcAbnormalFacade qcAbnormalFacade;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-同步前一天的入院记录
     */
    public void executeBehospital() {
        //String sql="select * from br_inpatientinfo where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        String sql="select * from br_inpatientinfo where cjcxrq>=(select CONVERT(varchar,GETDATE()-2,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)) ORDER BY cjcxrq DESC";
        List<BehospitalInfo> behospitalInfoList = tzDBConn.getBehospitalInfo(sql);
        execute(behospitalInfoList);
    }

    /**
     * 同步前一天的入院记录
     */
    public void executeBehospitalPast() {
        String sql="select * from br_inpatientinfo where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        List<BehospitalInfo> behospitalInfoList = tzDBConn.getBehospitalInfo(sql);
        execute(behospitalInfoList);
    }


    /**
     * 通过接口更新病人住院记录信息
     * @param list
     * @return
     */
    public RespDTO<List<ABehospitalInfoDTO>> executeBehospital(List<ABehospitalInfoVO> list) {
        if(list!=null && list.size()>0) {
            List<BehospitalInfo> behospitalInfoList = Lists.newArrayList();
            List<ABehospitalInfoDTO> behospitalInfoDtoList = Lists.newArrayList();
            list.stream().forEach(s->{
                BehospitalInfo behospitalInfo=new BehospitalInfo();
                BeanUtil.copyProperties(s,behospitalInfo);
                if(StringUtil.isNotBlank(s.getBirthday())) {
                    behospitalInfo.setBirthday(DateUtil.parseDate(s.getBirthday()));
                }
                if(StringUtil.isNotBlank(s.getLeaveHospitalDate())) {
                    behospitalInfo.setLeaveHospitalDate(DateUtil.parseDateTime(s.getLeaveHospitalDate()));
                }
                if(StringUtil.isNotBlank(s.getBehospitalDate())){
                    behospitalInfo.setBehospitalDate(DateUtil.parseDateTime(s.getBehospitalDate()));
                }
                if(StringUtil.isNotBlank(s.getPlacefileDate())){
                    behospitalInfo.setPlacefileDate(DateUtil.parseDateTime(s.getPlacefileDate()));
                }
                behospitalInfoList.add(behospitalInfo);

            });

            //循环验证数据有效性
            for (BehospitalInfo behospitalInfo:behospitalInfoList) {
                if("".equals(behospitalInfo.getBehospitalCode())) {
                    return RespDTO.onError("请输入病人住院编码!");
                }else if(behospitalInfo.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(behospitalInfo.getBehDeptId())){
                    return RespDTO.onError("请输入住院科室编码!");
                }else if("".equals(behospitalInfo.getBehDeptName())){
                    return RespDTO.onError("请输入住院科室名称!");
                }
            }
            execute(behospitalInfoList);
            behospitalInfoDtoList = BeanUtil.listCopyTo(list, ABehospitalInfoDTO.class);
            return RespDTO.onSuc(behospitalInfoDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<BehospitalInfo> behospitalInfoList){
        List<BehospitalInfo> addE = Lists.newLinkedList();
        List<BehospitalInfo> updateE = Lists.newLinkedList();
        behospitalInfoList.stream().forEach(s -> {
            BehospitalInfo behospitalInfo = this.getOne(new QueryWrapper<BehospitalInfo>()
                    .eq("behospital_code", s.getBehospitalCode())
                    .eq("hospital_id", s.getHospitalId())
                    .eq("is_deleted",IsDeleteEnum.N), false);
            if (behospitalInfo != null) {
                s.setGmtModified(new Date());
                s.setQcTypeId(initQcTypeId(s));
                updateE.add(s);
            } else {
                s.setGmtCreate(new Date());
                s.setQcTypeId(initQcTypeId(s));
                addE.add(s);
            }
        });
        if(addE.size()>0){
            behospitalInfoService.saveBatch(addE);
        }
        if(updateE.size()>0){
            behospitalInfoService.updateBatchByKey(updateE);
        }

    }

    /**
     * 初始化QcTypeId
     * @param s
     * @return
     */
    private Long initQcTypeId(BehospitalInfo s){
        Long qcTypeId=Long.valueOf("0");
        MedicalRecord medicalRecord = aMedicalRecordFacade.getOne(new QueryWrapper<MedicalRecord>()
                .eq("behospital_code", s.getBehospitalCode())
                .eq("hospital_id", s.getHospitalId())
                .eq("mode_id", Long.valueOf("1"))
                .eq("is_deleted",IsDeleteEnum.N), false);
        if(medicalRecord!=null){
            QcType qcType=qcTypeFacade.getOne(new QueryWrapper<QcType>()
                    .eq("name",medicalRecord.getRecTitle())
                    .eq("hospital_id", s.getHospitalId())
                    .eq("is_deleted",IsDeleteEnum.N), false);
            if(qcType!=null){
                qcTypeId=qcType.getId();
            }else{
                //新增类型到qc_type表中
                QcType qctype=new QcType();
                qctype.setHospitalId(s.getHospitalId());
                qctype.setName(medicalRecord.getRecTitle());
                qctype.setGmtCreate(new Date());

                Long id=qcTypeFacade.saveT(qctype);
                qcTypeId=id;

                //新增类型到监测表中
                QcAbnormal qcAbnormal=new QcAbnormal();
                qcAbnormal.setHospitalId(s.getHospitalId());
                qcAbnormal.setBehospitalCode(s.getBehospitalCode());
                qcAbnormal.setType(2);
                qcAbnormal.setDescription(medicalRecord.getRecTitle());
                qcAbnormal.setGmtCreate(new Date());
                qcAbnormalFacade.getBaseMapper().insert(qcAbnormal);
            }
        }
        return qcTypeId;
    }
}
