package com.diagbot.facade.data;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.AHomeDiagnoseDTO;
import com.diagbot.entity.HomeDiagnoseInfo;
import com.diagbot.service.impl.HomeDiagnoseInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.AHomeDiagnoseVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class AHomeDiagnoseInfoFacade extends HomeDiagnoseInfoServiceImpl{

    @Autowired
    @Qualifier("homeDiagnoseInfoServiceImpl")
    private HomeDiagnoseInfoServiceImpl homeDiagnoseInfoService;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-定时同步病案首页诊断
     */
    public void executeHomeDiagnose() {
        //String sql="select * from br_recdiagnose where BASYID in (select BASYID from br_rechome where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate())";//今年的数据
        //前一天的数据
        String sql="select * from br_recdiagnose where BASYID in (select BASYID from br_rechome where cjcxrq>=(select CONVERT(varchar,GETDATE()-1,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)))";
        List<HomeDiagnoseInfo> homeDiagnoseList = tzDBConn.getHomeDiagnose(sql);
        execute(homeDiagnoseList);
    }

    /**
     * 数据引擎-通过接口更新病案首页诊断
     * @param list
     * @return
     */
    public RespDTO<List<AHomeDiagnoseDTO>> executeHomeDiagnose(List<AHomeDiagnoseVO> list) {
        if(list!=null && list.size()>0){
            List<HomeDiagnoseInfo> homeDiagnoseList=Lists.newArrayList();
            List<AHomeDiagnoseDTO> homeDiagnoseDtoList=Lists.newArrayList();

            homeDiagnoseList=BeanUtil.listCopyTo(list,HomeDiagnoseInfo.class);
            homeDiagnoseDtoList=BeanUtil.listCopyTo(list,AHomeDiagnoseDTO.class);
            execute(homeDiagnoseList);
            return RespDTO.onSuc(homeDiagnoseDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<HomeDiagnoseInfo> homeDiagnoseList){
        List<HomeDiagnoseInfo> addE = Lists.newLinkedList();
        List<HomeDiagnoseInfo> updateE = Lists.newLinkedList();
        if (homeDiagnoseList != null && homeDiagnoseList.size() > 0) {
            homeDiagnoseList.stream().forEach(s -> {
                if(s.getHomePageId()!=null && !"".equals(s.getDiagnoseOrderNo())){
                    HomeDiagnoseInfo diagnoseInfo = this.getOne(new QueryWrapper<HomeDiagnoseInfo>()
                            .eq("home_page_id", s.getHomePageId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("diagnose_order_no", s.getDiagnoseOrderNo()), false);
                    if (diagnoseInfo != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                }
            });
        }
        if(addE.size()>0){
            homeDiagnoseInfoService.saveBatch(addE);
        }
        if(updateE.size()>0){
            homeDiagnoseInfoService.updateBatchByKey(updateE);
        }
    }
}
