package com.diagbot.facade.data;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.AMedCrisisDTO;
import com.diagbot.entity.MedCrisisInfo;
import com.diagbot.service.impl.MedCrisisInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.vo.data.AMedCrisisVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class AMedCrisisFacade extends MedCrisisInfoServiceImpl{

    @Autowired
    @Qualifier("medCrisisInfoServiceImpl")
    private MedCrisisInfoServiceImpl medCrisisInfoService;

    public RespDTO<List<AMedCrisisDTO>> executeMedCrisis(List<AMedCrisisVO> list) {
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (AMedCrisisVO aMedCrisisVO:list) {
                if("".equals(aMedCrisisVO.getRecId())){
                    return RespDTO.onError("请输入文书编码!");
                }else if("".equals(aMedCrisisVO.getBehospitalCode())) {
                    return RespDTO.onError("请输入病人住院编码!");
                }else if(aMedCrisisVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(aMedCrisisVO.getRecType())){
                    return RespDTO.onError("请输入危急值类型!");
                }
            }

            List<MedCrisisInfo> medCrisisList=Lists.newArrayList();
            list.stream().forEach(s->{
                MedCrisisInfo medCrisisInfo=new MedCrisisInfo();
                BeanUtil.copyProperties(s,medCrisisInfo);
                medCrisisInfo.setRepDate(DateUtil.parseDateTime(s.getRepDate()));
                medCrisisInfo.setSendDate(DateUtil.parseDateTime(s.getSendDate()));
                medCrisisList.add(medCrisisInfo);
            });
            execute(medCrisisList);

            List<AMedCrisisDTO> medCrisisDtoList=BeanUtil.listCopyTo(list,AMedCrisisDTO.class);
            return RespDTO.onSuc(medCrisisDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<MedCrisisInfo> medCrisisList){
        List<MedCrisisInfo> addE = Lists.newLinkedList();
        List<MedCrisisInfo> updateE = Lists.newLinkedList();
        if (medCrisisList != null && medCrisisList.size() > 0) {
            medCrisisList.stream().forEach(s -> {
                MedCrisisInfo medCrisisInfo = this.getOne(new QueryWrapper<MedCrisisInfo>()
                .eq("rec_id", s.getRecId())
                .eq("hospital_id", s.getHospitalId())
                .eq("behospital_code", s.getBehospitalCode())
                .eq("rec_type", s.getRecType()), false);
                if (medCrisisInfo != null) {
                    s.setGmtModified(new Date());
                    updateE.add(s);
                } else {
                    s.setGmtCreate(new Date());
                    addE.add(s);
                }
            });
        }
        if(addE.size()>0){
            medCrisisInfoService.saveBatch(addE);
        }
        if(updateE.size()>0){
            medCrisisInfoService.updateBatchByKey(updateE);
        }
    }
}
