package com.diagbot.facade.data;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.AnalyzeRunDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.AMedicalRecordDTO;
import com.diagbot.dto.data.AMrContentDTO;
import com.diagbot.entity.*;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.facade.BehospitalInfoFacade;
import com.diagbot.facade.ModelHospitalFacade;
import com.diagbot.facade.QcTypeFacade;
import com.diagbot.service.impl.MedicalRecordServiceImpl;
import com.diagbot.service.impl.QcAbnormalServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ReadProperties;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.AnalyzeRunVO;
import com.diagbot.vo.data.AMedicalRecordVO;
import com.diagbot.vo.data.AMrContentVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class AMedicalRecordFacade extends MedicalRecordServiceImpl {

    @Autowired
    @Qualifier("medicalRecordServiceImpl")
    private MedicalRecordServiceImpl medicalRecordService;

    @Autowired
    private ModelHospitalFacade modelHospitalFacade;

    @Autowired
    private AMedicalRecordContentFacade aMedicalRecordContentFacade;

    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    @Autowired
    @Qualifier("qcAbnormalServiceImpl")
    private QcAbnormalServiceImpl qcAbnormalService;

    @Autowired
    private QcTypeFacade qcTypeFacade;

    @Autowired
    private ReadProperties readProperties;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-同步前一天的病历记录
     */
    public void executeMrRecord() {
        //String sql="select * from mr_medicalrecords  where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        String sql="select * from mr_medicalrecords  where cjcxrq>=(select CONVERT(varchar,GETDATE()-2,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)) ORDER BY cjcxrq DESC";
        List<MedicalRecord> medicalRecordList = tzDBConn.getMedicalRecord(sql);
        execute(medicalRecordList);
    }

    /**
     * 终末质控-通过接口更新病历记录
     * @param list
     * @return
     */
    public RespDTO<List<AMedicalRecordDTO>> executeMrRecord(List<AMedicalRecordVO> list) {
        if(list!=null && list.size()>0) {
            //循环验证数据有效性
            for (AMedicalRecordVO aMedicalRecordVO : list) {
                if ("".equals(aMedicalRecordVO.getRecId())) {
                    return RespDTO.onError("请输入文书序号!");
                } else if (aMedicalRecordVO.getHospitalId() == null) {
                    return RespDTO.onError("请输入医院编码!");
                } else if ("".equals(aMedicalRecordVO.getBehospitalCode())) {
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }

            //装载数据
            List<MedicalRecord> medicalRecordList = Lists.newArrayList();
            list.stream().forEach(s->{
                MedicalRecord medicalRecord=new MedicalRecord();
                BeanUtil.copyProperties(s,medicalRecord);
                medicalRecord.setRecDate(DateUtil.parseDateTime(s.getRecDate()));
                medicalRecordList.add(medicalRecord);
            });
            execute(medicalRecordList);

            List<AMedicalRecordDTO> medicalRecordDTOList = BeanUtil.listCopyTo(list, AMedicalRecordDTO.class);
            return RespDTO.onSuc(medicalRecordDTOList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    /**
     * 运行质控-通过接口更新病历记录
     * @param aMrContentVO
     * @return
     */
    public RespDTO<Map<String, Object>> executeMrRecordIng(AMrContentVO aMrContentVO) {
        if(aMrContentVO.getRecords()!=null && aMrContentVO.getRecords().size()>0){
            AMrContentDTO aMrContentDTO=new AMrContentDTO();
            //循环验证数据有效性
            for (AMedicalRecordVO aMedicalRecordVO : aMrContentVO.getRecords()) {
                if ("".equals(aMedicalRecordVO.getRecId())) {
                    return RespDTO.onError("请输入文书序号!");
                } else if (aMedicalRecordVO.getHospitalId() == null) {
                    return RespDTO.onError("请输入医院编码!");
                } else if ("".equals(aMedicalRecordVO.getBehospitalCode())) {
                    return RespDTO.onError("请输入病人住院编码!");
                } else if ("".equals(aMedicalRecordVO.getRecTypeId())) {
                    return RespDTO.onError("请输入病历类别编号!");
                } else if ("".equals(aMedicalRecordVO.getRecTitle())) {
                    return RespDTO.onError("请输入病历标题!");
                } else if (aMedicalRecordVO.getContents() == null || aMedicalRecordVO.getContents().size() == 0) {
                    return RespDTO.onError("请输入文书详情!");
                }
            }
            //更新文书信息
            executeMrRecord(aMrContentVO.getRecords());

            aMrContentVO.getRecords().stream().forEach(s -> {
                //更新文书详情
                aMedicalRecordContentFacade.executeMrRecordContent(s.getContents());
            });

            //评分后返回结构体
            return mrIng(aMrContentVO);

        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    /**
     * 评分后返回结构体
     * @param aMrContentVO
     * @return
     */
    private RespDTO<Map<String, Object>> mrIng(AMrContentVO aMrContentVO){
        String behospitalCode =aMrContentVO.getRecords().get(0).getBehospitalCode();
        Long hospitalId=aMrContentVO.getRecords().get(0).getHospitalId();
        MedicalRecord medicalRecord=new MedicalRecord();
        BeanUtil.copyProperties(aMrContentVO.getRecords().get(0),medicalRecord);
        Long modeId=initModeId(medicalRecord);
        if(aMrContentVO.getDockModeType().equals("0")){
            Map<String,Object> map=new HashMap<String,Object>();
            List<AMedicalRecordDTO> medicalRecordDTOList = BeanUtil.listCopyTo(aMrContentVO.getRecords(), AMedicalRecordDTO.class);
            map.put("records",medicalRecordDTOList);
            return RespDTO.onSuc(map);
        }else if(aMrContentVO.getDockModeType().equals("1")){
            if(modeId==0 || modeId==null){
                return RespDTO.onError("无对应的文书类型");
            }
            //页面模式
            String url=readProperties.getProcessQcUrl()+"?behospitalCode="+behospitalCode+"&hospitalId="+hospitalId+"&modeId="+modeId;
            Map<String,Object> map=new HashMap<String,Object>();
            map.put("url",url);
            return RespDTO.onSuc(map);
        }else if(aMrContentVO.getDockModeType().equals("2")){
            AnalyzeRunVO analyzeRunVO=new AnalyzeRunVO();
            analyzeRunVO.setBehospitalCode(behospitalCode);
            analyzeRunVO.setHospitalId(hospitalId);
            analyzeRunVO.setModeId(modeId);
            AnalyzeRunDTO analyzeRunDTO=behospitalInfoFacade.analyzeRun(analyzeRunVO);
            Map<String,Object> map=new HashMap<String,Object>();
            if(analyzeRunDTO.getMsgDTOList().size()>0){
                map.put("analyze",analyzeRunDTO.getMsgDTOList());
            }
            //接口引擎模式
            return RespDTO.onSuc(map);
        }else{
            if(modeId==0 || modeId==null){
                return RespDTO.onError("无对应的文书类型");
            }
            //页面模式
            String url=readProperties.getProcessQcUrl()+"?behospitalCode="+behospitalCode+"&hospitalId="+hospitalId+"&modeId="+modeId;

            AnalyzeRunVO analyzeRunVO=new AnalyzeRunVO();
            analyzeRunVO.setBehospitalCode(behospitalCode);
            analyzeRunVO.setHospitalId(hospitalId);
            analyzeRunVO.setModeId(modeId);
            AnalyzeRunDTO analyzeRunDTO=behospitalInfoFacade.analyzeRun(analyzeRunVO);
            Map<String,Object> map=new HashMap<String,Object>();
            if(analyzeRunDTO.getMsgDTOList().size()>0){
                map.put("url",url);
                map.put("analyze",analyzeRunDTO.getMsgDTOList());
            }
            return RespDTO.onSuc(map);
        }
    }

    public void execute(List<MedicalRecord> medicalRecordList){
        List<MedicalRecord> addE = Lists.newLinkedList();
        List<MedicalRecord> updateE = Lists.newLinkedList();
        List<BehospitalInfo> behospitalInfoList=Lists.newLinkedList();
        List<QcAbnormal> qcAbnormalList = Lists.newLinkedList();
        if (medicalRecordList != null && medicalRecordList.size() > 0) {
            medicalRecordList.stream().forEach(s -> {
                //初始化mode_id
                Long modeId=initModeId(s);
                if(modeId==Long.valueOf("0")){
                    //新增类型到监测表中
                    QcAbnormal qcAbnormal=new QcAbnormal();
                    qcAbnormal.setHospitalId(s.getHospitalId());
                    qcAbnormal.setBehospitalCode(s.getBehospitalCode());
                    qcAbnormal.setType(1);
                    qcAbnormal.setDescription(s.getRecTitle());
                    qcAbnormal.setGmtCreate(new Date());
                    qcAbnormalList.add(qcAbnormal);
                }else if(modeId==Long.valueOf("1")){

                    BehospitalInfo behospitalInfo = behospitalInfoFacade.getOne(new QueryWrapper<BehospitalInfo>()
                            .eq("behospital_code", s.getBehospitalCode())
                            .eq("hospital_id", s.getHospitalId()), false);
                    if(behospitalInfo!=null){
                        //如果病人住院信息存在，更新对应的qc_type_id
                        Long qcTypeId=Long.valueOf("0");
                        QcType qcType=qcTypeFacade.getOne(new QueryWrapper<QcType>()
                                .eq("name",s.getRecTitle())
                                .eq("hospital_id", s.getHospitalId())
                                .eq("is_deleted",IsDeleteEnum.N), false);
                        if(qcType!=null){
                            behospitalInfo.setQcTypeId(qcType.getId());
                        }else{
                            //新增类型到qc_type表中
                            QcType qctype=new QcType();
                            qctype.setHospitalId(s.getHospitalId());
                            qctype.setName(s.getRecTitle());
                            qctype.setGmtCreate(new Date());

                            Long id=qcTypeFacade.saveT(qctype);
                            behospitalInfo.setQcTypeId(id);

                            //新增类型到监测表中
                            QcAbnormal qcAbnormal=new QcAbnormal();
                            qcAbnormal.setHospitalId(s.getHospitalId());
                            qcAbnormal.setBehospitalCode(s.getBehospitalCode());
                            qcAbnormal.setType(2);
                            qcAbnormal.setDescription(s.getRecTitle());
                            qcAbnormal.setGmtCreate(new Date());
                            qcAbnormalService.getBaseMapper().insert(qcAbnormal);
                        }
                        behospitalInfoList.add(behospitalInfo);
                    }
                }
                s.setModeId(modeId);
                //新增或修改
                MedicalRecord mRecord = this.getOne(new QueryWrapper<MedicalRecord>()
                        .eq("rec_id", s.getRecId())
                        .eq("hospital_id", s.getHospitalId())
                        .eq("behospital_code", s.getBehospitalCode()), false);
                if (mRecord != null) {
                    s.setGmtModified(new Date());
                    updateE.add(s);
                } else {
                    s.setGmtCreate(new Date());
                    addE.add(s);
                }
            });
        }
        if(addE.size()>0){
            medicalRecordService.saveBatch(addE);
        }
        if(updateE.size()>0){
            medicalRecordService.updateBatchByKey(updateE);
        }
        if(qcAbnormalList.size()>0){
            qcAbnormalService.saveBatch(qcAbnormalList);
        }
        if(behospitalInfoList.size()>0){
            behospitalInfoFacade.updateBatchByKey(behospitalInfoList);
        }
    }


    /**
     * 初始化模型ID
     * @param medicalRecord
     * @return
     */
    private Long initModeId(MedicalRecord medicalRecord){
        Long modeId=Long.valueOf("0");
        QueryWrapper<ModelHospital> wrapper = new QueryWrapper<>();
        wrapper.eq("hospital_id",medicalRecord.getHospitalId());
        wrapper.eq("hospital_model_name",medicalRecord.getRecTitle());
        ModelHospital mode=modelHospitalFacade.getOne(wrapper, false);
        if(mode!=null){
            modeId=mode.getStandModelId();
        }else{
            modeId=Long.valueOf("0");
        }
        return modeId;
    }

}
